import javax.swing.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Component;
import java.awt.event.*;
import java.io.*;
import javax.swing.text.BadLocationException;
import java.util.Properties;

public class MainFrame extends JFrame implements ActionListener {
	
	private static Editeur editeur;
	private static BarreDeStatut barreDeStatut;
	private static JMenuItem[] menuFichier;
	private static JMenuItem[] menuEdition;
	private static JMenuItem[] menuAnalyse;
	private static JMenuItem[] menuTraduction;
	private static JMenuItem[] menuExecution;
	
	// Algorithme charg dans l'diteur
	private File fichierAlg;
	// Dernire traduction de l'algorithme courant
	private File fichierTraduit;
	// true si l'algorithme a t modifi dans l'diteur
	private boolean modifie;
	// Dtection de Galg et des langages
	private DetectionOutils detection;
	
	private static final int TAILLE_FICHIER = 5;
	private static final int TAILLE_EDITION = 7;
	private static final int TAILLE_ANALYSE = 4;
	private static final int TAILLE_TRADUCTION = 9; 
	private static final int TAILLE_EXECUTION = 8;
	
	// Constantes pour accder aux entres des menus
	// Fichier
	private static final int NOUVEAU = 0;
	private static final int OUVRIR = 1;
	private static final int ENREGISTRER = 2;
	private static final int ENREGISTRER_SOUS = 3;
	private static final int QUITTER = 4;
	// Edition 
	private static final int ALLER_ERREUR = 0;
	private static final int ALLER_LIGNE = 1;
	private static final int INSERER_AFFECTATION = 2;
	private static final int INSERER_COMMENTAIRE = 3;
	private static final int INSERER_POUR = 4;
	private static final int INSERER_SI = 5;
	private static final int INSERER_TANT_QUE = 6;
	// Analyse
	private static final int LANCER_ANALYSE = 0;
	private static final int VOIR_ALGO = 1;
	private static final int VOIR_RES = 2;
	private static final int VOIR_VAR = 3;
	// Traduction
	private static final int TRADUCTION_C = 0;
	private static final int TRADUCTION_CPP = 1;
	private static final int TRADUCTION_DBASE = 2;
	private static final int TRADUCTION_JAVA = 3;
	private static final int TRADUCTION_PASCAL = 4;
	private static final int TRADUCTION_PERL = 5;
	private static final int TRADUCTION_REXX = 6;
	private static final int TRADUCTION_TCL = 7;
	private static final int VOIR_TRADUCTION = 8;
	// Excution
	private static final int EXECUTION_C = 0;
	private static final int EXECUTION_CPP = 1;
	private static final int EXECUTION_DBASE = 2;
	private static final int EXECUTION_JAVA = 3;
	private static final int EXECUTION_PASCAL = 4;
	private static final int EXECUTION_PERL = 5;
	private static final int EXECUTION_REXX = 6;
	private static final int EXECUTION_TCL = 7;
	
	public MainFrame (String titre) {
		super (titre);
		setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
		menuFichier=new JMenuItem[TAILLE_FICHIER];
		menuEdition=new JMenuItem[TAILLE_EDITION];
		menuAnalyse=new JMenuItem[TAILLE_ANALYSE];
		menuTraduction=new JMenuItem[TAILLE_TRADUCTION];
		menuExecution=new JMenuItem[TAILLE_EXECUTION];
		modifie=false;
		detection = new DetectionOutils ();
		construitMenu ();
//		barre d'tat
		barreDeStatut = new BarreDeStatut (getContentPane());
		GridBagLayout layout = new GridBagLayout ();
		editeur=new Editeur (this,barreDeStatut);
		JScrollPane editeurScroller = new JScrollPane (editeur);
		getContentPane().setLayout (layout);	
		// Ajout des composants
		nouveauComposant (editeurScroller, layout,0,0,1,1,GridBagConstraints.BOTH,GridBagConstraints.NORTHWEST);
		nouveauComposant (barreDeStatut, layout,0,1,1,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.SOUTH);
		setLocationRelativeTo (null);
		setSize (400,400);
		setVisible (true);
	}
	
	// Accesseurs
	public void setModifie (boolean modifie) {
		this.modifie=modifie;
	}
	
	public boolean getModifie () {
		return modifie;
	}
	
	public String getFichierAlg () {
		if (fichierAlg!=null)
			return fichierAlg.getName();
		else
			return "Nouvel algorithme";
	}
	
	/*
	 * 	Ajout d'un composant avec contraintes pour son placement
	 */
	
	private void nouveauComposant (Component composant, GridBagLayout layout, int gridx, int gridy, int weightx, int weighty, int fill, int anchor) {
		GridBagConstraints gc = new GridBagConstraints ();
		gc.gridx=gridx;
		gc.gridy=gridy;
		gc.weightx=weightx;
		gc.weighty=weighty;
		gc.fill=fill;
		gc.anchor=anchor;
		layout.setConstraints (composant,gc);
		getContentPane().add(composant);
	}
	
	/* 
	 *		Cration du menu
	 */
	
	private void construitMenu () {
		JMenuBar galgMenuBar = new JMenuBar();
		// Menu "Fichier"
		JMenu galgMenu = new JMenu ("Fichier");
		galgMenu.add (menuFichier[NOUVEAU]=creerNouvelItem("Nouveau","nouveau",true));
		galgMenu.add (menuFichier[OUVRIR]=creerNouvelItem("Ouvrir un algorithme...","ouvrir",true));	
		galgMenu.add (menuFichier[ENREGISTRER]=creerNouvelItem("Enregistrer","enregistrer",true));
		galgMenu.add (menuFichier[ENREGISTRER_SOUS]=creerNouvelItem ("Enregistrer sous ...","enregistrer_sous",true));
		galgMenu.add (menuFichier[QUITTER]=creerNouvelItem("Quitter","quitter",true));
		galgMenuBar.add(galgMenu);
		// Menu Edition
		galgMenu = new JMenu ("Edition");
		galgMenu.add (menuEdition[ALLER_ERREUR]=creerNouvelItem("Aller  la premire erreur dtecte","aller_premiere_erreur",false));
		galgMenu.add (menuEdition[ALLER_LIGNE]=creerNouvelItem("Aller  la ligne numro ...","aller_ligne",true));
		galgMenu.add (menuEdition[INSERER_AFFECTATION]=creerNouvelItem ("Insrer une affectation","inserer_affectation",true));
		galgMenu.add (menuEdition[INSERER_COMMENTAIRE]=creerNouvelItem("Insrer un commentaire","inserer_commentaire",true));
		galgMenu.add (menuEdition[INSERER_POUR]=creerNouvelItem("Insrer une boucle pour","inserer_boucle_pour",true));
		galgMenu.add (menuEdition[INSERER_SI]=creerNouvelItem("Insrer un test si...alors...sinon","inserer_test_si",true));
		galgMenu.add (menuEdition[INSERER_TANT_QUE]=creerNouvelItem("Insrer une boucle \"tant que\"","inserer_boucle_tant_que",true));
		galgMenuBar.add (galgMenu); 
		// Menu Analyse
		galgMenu = new JMenu ("Analyse");
		galgMenu.add (menuAnalyse[LANCER_ANALYSE]=creerNouvelItem("Lancer l'analyse","lancer_analyse",true));
		galgMenu.add (menuAnalyse[VOIR_ALGO]=creerNouvelItem("Voir l'algorithme","voir_algo",false));
		galgMenu.add (menuAnalyse[VOIR_RES]=creerNouvelItem("Voir le fichier rsultat","voir_res",false));
		galgMenu.add (menuAnalyse[VOIR_VAR]=creerNouvelItem("Voir le fichier des variables","voir_var",false));
		galgMenuBar.add (galgMenu);
		// Menu Traduction
		galgMenu = new JMenu ("Traduction");
		boolean enable = (detection.getVersion(DetectionOutils.GALG)!=null);
		galgMenu.add (menuTraduction[TRADUCTION_C]=creerNouvelItem("C","traduire_c",enable));
		galgMenu.add (menuTraduction[TRADUCTION_CPP]=creerNouvelItem("C++","traduire_cpp",enable));
		galgMenu.add (menuTraduction[TRADUCTION_DBASE]=creerNouvelItem("DBase","traduire_dbase",enable));
		galgMenu.add (menuTraduction[TRADUCTION_JAVA]=creerNouvelItem("Java","traduire_java",enable));
		galgMenu.add (menuTraduction[TRADUCTION_PASCAL]=creerNouvelItem("Pascal","traduire_pascal",enable));
		galgMenu.add (menuTraduction[TRADUCTION_PERL]=creerNouvelItem("Perl","traduire_perl",enable));
		galgMenu.add (menuTraduction[TRADUCTION_REXX]=creerNouvelItem("Rexx","traduire_rexx",enable));
		galgMenu.add (menuTraduction[TRADUCTION_TCL]=creerNouvelItem("Tcl","traduire_tcl",enable));
		galgMenu.add (menuTraduction[VOIR_TRADUCTION]=creerNouvelItem("Voir traduction","voir_traduction",false));
		galgMenuBar.add (galgMenu);
		// Menu Excution
		galgMenu = new JMenu ("Excution");
		galgMenu.add (menuExecution[EXECUTION_C]=creerNouvelItem("C","executer_c", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_C)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null) ));
		galgMenu.add (menuExecution[EXECUTION_CPP]=creerNouvelItem("C++","executer_cpp", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_CPP)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null) ));
		galgMenu.add (menuExecution[EXECUTION_DBASE]=creerNouvelItem("DBase","executer_dbase", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_DBASE)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenu.add (menuExecution[EXECUTION_JAVA]=creerNouvelItem("Java","executer_java", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_JAVA)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenu.add (menuExecution[EXECUTION_PASCAL]=creerNouvelItem("Pascal","executer_pascal", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_PASCAL)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenu.add (menuExecution[EXECUTION_PERL]=creerNouvelItem("Perl","executer_perl", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_PERL)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenu.add (menuExecution[EXECUTION_REXX]=creerNouvelItem("Rexx","executer_rexx",(detection.getVersion(DetectionOutils.LANG_REXX)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenu.add (menuExecution[EXECUTION_TCL]=creerNouvelItem("Tcl","executer_tcl", ((detection.getVersion(DetectionOutils.LANG_REXX)!=null)) &&  (detection.getVersion(DetectionOutils.LANG_TCL)!=null) && (detection.getVersion(DetectionOutils.GALG)!=null)));
		galgMenuBar.add (galgMenu);
		// Menu d'aide
		galgMenu = new JMenu ("?");
		galgMenu.add (creerNouvelItem("Aide sur GALG","aide_galg",true));
		galgMenu.add (creerNouvelItem("Langages supports","langages_supportes",true));
		galgMenu.add (creerNouvelItem("Langages prsents","langages_presents",true));
		galgMenu.add (creerNouvelItem("A propos ...","a_propos",true));
		galgMenuBar.add (galgMenu);
		setJMenuBar (galgMenuBar);
	}
	
	/*
	 *    Cration d'une entre de menu (Item)
	 */
	private JMenuItem creerNouvelItem (String texte, String commande, boolean enable) {
		JMenuItem menuItem = new JMenuItem (texte);
		menuItem.setEnabled (enable);
		menuItem.setActionCommand(commande);
		menuItem.addActionListener (this);
		return menuItem;
	}
	
	// Demande  l'utilisateur s'il veut sauver son travail
	private void sauvegarde () {
		if (modifie==true) {
			if (fichierAlg!=null) 
				enregistrer ();
			else {
				int choix = JOptionPane.showConfirmDialog(this, "L'algorithme a t modifi.\nVoulez-vous l'enregistrer ?","Voulez-vous sauvegarder ?",JOptionPane.YES_NO_OPTION);
				if (choix==JOptionPane.YES_OPTION)
					enregistrer_sous ();
			}
		}
	}
	
	private boolean ouvrir(File fichier) {
		try {
			BufferedReader inputStream = new BufferedReader (new FileReader (fichier));
			editeur.setText(""); // Efface l'ancien algorithme de l'diteur
			String buffer = inputStream.readLine();
			while (buffer!=null) {
				editeur.append(buffer+"\n");
				buffer = inputStream.readLine();
			}
			inputStream.close();
			// Ecrit le nom du fichier dans la barre d'tat
			barreDeStatut.setNomFichier(fichier.getName());
			return true;
		}
		catch (FileNotFoundException except) {
			// Impossible d'ouvrir le fichier slectionn
			JOptionPane.showMessageDialog(this,"Impossible d'ouvrir "+fichier.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			return false;
		}
		catch (IOException except) {
			// Erreur pendant la lecture
			JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant la lecture du fichier"+fichier.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
	
	private void enregistrer () {
		//		Vrifie qu'un algorithme est ouvert
		if (fichierAlg!=null) {
			try {
				BufferedWriter outputStream = new BufferedWriter (new FileWriter (fichierAlg));
				String buffer = editeur.getText();
				outputStream.write(buffer);
				outputStream.flush();
				outputStream.close();
				modifie=false;
				barreDeStatut.setNomFichier(fichierAlg.getName());
			}
			catch (IOException except){
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'enregistrement du fichier"+fichierAlg.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
			catch (NullPointerException except) {
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'enregistrement du fichier"+fichierAlg.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	private void voirRes () {
		String Lst = new String (fichierAlg.toString());
		Lst=Lst.replaceFirst("[.]alg",".lst");
		// On affiche le fichier lst
		File lst=new File (Lst);
		if (ouvrir (lst)) {
			editeur.setEditable(false);
			// On rend accessible les fonctions ncessaires du menu
			menuFichier[ENREGISTRER].setEnabled(false);
			menuFichier[ENREGISTRER_SOUS].setEnabled(false);
			menuAnalyse[VOIR_ALGO].setEnabled (true);
			menuAnalyse[VOIR_RES].setEnabled(false);
			menuAnalyse[VOIR_VAR].setEnabled(true);
			for (int i=ALLER_LIGNE;i<=INSERER_TANT_QUE;i++)
				menuEdition[i].setEnabled(false);
			if (fichierTraduit==null)
				menuTraduction[VOIR_TRADUCTION].setEnabled(false);
			else
				menuTraduction[VOIR_TRADUCTION].setEnabled(true);
			for (int i=ALLER_LIGNE;i<=INSERER_TANT_QUE;i++)
							menuEdition[i].setEnabled(false);
			menuEdition[ALLER_ERREUR].setEnabled(true);
		}
		modifie=false;
	}
	
	private void voirAlgo () {
		if (ouvrir (fichierAlg)) {
			// On met les menus  jour
			menuFichier[ENREGISTRER].setEnabled(true);
			menuFichier[ENREGISTRER_SOUS].setEnabled(true);
			menuAnalyse[VOIR_ALGO].setEnabled(false);
			menuAnalyse[VOIR_RES].setEnabled(true);
			menuAnalyse[VOIR_VAR].setEnabled(true);
			for (int i=ALLER_LIGNE;i<=INSERER_TANT_QUE;i++)
				menuEdition[i].setEnabled(true);				
			// On autorise les modifications dans l'diteur
			editeur.setEditable(true);
			if (fichierTraduit==null)
				menuTraduction[VOIR_TRADUCTION].setEnabled(false);
			else
				menuTraduction[VOIR_TRADUCTION].setEnabled(true);
			for (int i=ALLER_ERREUR;i<=INSERER_TANT_QUE;i++)
				menuEdition[i].setEnabled(true);
		}
		modifie=false;		
	}
	
	private void enregistrer_sous () {
		JFileChooser fileChooser = new JFileChooser ();
		fileChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
		fileChooser.setAcceptAllFileFilterUsed(false);
		FiltreAlgo filtre = new FiltreAlgo ();
		fileChooser.setFileFilter (filtre);
		int retour=fileChooser.showSaveDialog(this);
		if (retour==JFileChooser.APPROVE_OPTION) {
			File fichier = fileChooser.getSelectedFile();
			if (fichier.exists()) {
				int choix = JOptionPane.showConfirmDialog (this, "Le fichier "+fichier.getName()+" existe dja.\nVoulez-vous l'craser ?", "Confirmation d'enregistrement", JOptionPane.YES_NO_OPTION);
				if (choix==JOptionPane.NO_OPTION)	
					return;
			}
			try {
				BufferedWriter outputStream = new BufferedWriter (new FileWriter (fichier));
				String buffer = editeur.getText();
				outputStream.write(buffer);
				outputStream.flush();
				outputStream.close();
				fichierAlg=fichier;
				modifie=false;
				barreDeStatut.setNomFichier(fichierAlg.getName());
			}
			catch (IOException IOexcept) {
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'enregistrement du fichier"+fichierAlg.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
			catch (NullPointerException except) {
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'enregistrement du fichier"+fichierAlg.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
		}
	}
	
	/*
	 *   	Gestion des commandes du menu
	 */
	public void actionPerformed (ActionEvent event) {
		String cmd = event.getActionCommand();
		if (cmd.equals("quitter")) {
			if (modifie==true) {
			int choix = JOptionPane.showConfirmDialog(this, "L'algorithme a t modifi.\nVoulez-vous l'enregistrer ?","Voulez-vous sauvegarder ?",JOptionPane.YES_NO_OPTION);
				if (choix==JOptionPane.YES_OPTION) {
					if (fichierAlg!=null)
						enregistrer_sous ();
					else
						enregistrer ();
				}
			dispose ();
			}	
			else
				dispose ();
		}
		else if (cmd.equals("langages_supportes")) {
			String message = new String ("- C\n- C++\n- DBase\n- Java\n- Pascal\n- Perl\n- Rexx\n- Tcl");
			JOptionPane.showMessageDialog (this,message,"Langages supports",JOptionPane.INFORMATION_MESSAGE);
		}
		else if (cmd.equals("aide_galg")) {
			Properties proprietes = System.getProperties();
			String os =	proprietes.getProperty ("os.name");
			if (os.equals("Linux")) {
				try {
					Runtime.getRuntime().exec("mozilla http://www.info.univ-angers.fr/pub/gh/Galg/galg.htm");
				}
				catch (IOException except) {
					JOptionPane.showMessageDialog (this,"Veuillez consulter l'aide sur\nhttp://www.info.univ-angers.fr/pub/gh/Galg/galg.htm","Aide de GALG",JOptionPane.INFORMATION_MESSAGE);
				}
			}
			else {
				try {
					Runtime.getRuntime().exec("start http://www.info.univ-angers.fr/pub/gh/Galg/galg.htm");
				}
				catch (IOException except) {
					JOptionPane.showMessageDialog (this,"Veuillez consulter l'aide sur\nhttp://www.info.univ-angers.fr/pub/gh/Galg/galg.htm","Aide de GALG",JOptionPane.INFORMATION_MESSAGE);
				}	
			}
		}
		else if (cmd.equals("a_propos")) {
			String galgMessage;
			if (detection.getVersion(DetectionOutils.GALG)!=null)
				galgMessage=new String ("GALG v"+detection.getVersion(DetectionOutils.GALG)+" dtect");
			else
				galgMessage=new String ("GALG non dtect");
			String perlMessage;
			if (detection.getVersion(DetectionOutils.LANG_PERL)!=null)
				perlMessage=new String ("Perl v"+detection.getVersion(DetectionOutils.LANG_PERL)+" dtect");
			else
				perlMessage=new String ("Perl non dtect");
			JOptionPane.showMessageDialog (this,"Interface pour GALG.\nRalise par Tran Sophie et Lvy Romain sous la direction M.Hunault Gilles.\n\n"+perlMessage+"\n"+galgMessage,"A propos ...",JOptionPane.INFORMATION_MESSAGE);
		}
		else if (cmd.equals("langages_presents")) {
			String langages[] = {"C : gcc ","C++ : g++ ","DBase ","Java : java ","Pascal : ppc386 ","Perl : perl ","Rexx : regina ","Tcl : tclsh "};
			String message=new String();
			for (int i=DetectionOutils.LANG_C;i<=DetectionOutils.LANG_TCL;i++) {
				String version;
				if ((version=detection.getVersion(i))!=null)
					message+=langages[i]+"v"+version+"\n";
			} 
			JOptionPane.showMessageDialog (this,message,"Langages prsents",JOptionPane.INFORMATION_MESSAGE);
		}
		else if (cmd.equals("ouvrir")) {
			sauvegarde ();
			JFileChooser fileChooser = new JFileChooser ();
			fileChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
			fileChooser.setAcceptAllFileFilterUsed(false);
			FiltreAlgo filtre = new FiltreAlgo ();
			fileChooser.setFileFilter (filtre);
			int retour = fileChooser.showOpenDialog(this);
			if (retour==JFileChooser.APPROVE_OPTION) {  // L'utilisateur a cliqu sur "Ok"
				File fichier = fileChooser.getSelectedFile();
				if (ouvrir (fichier)) {
					fichierAlg=fichier;
					modifie=false;
					// L'analyse n'est pas encore faite, on grise Edition->Aller  la premire erreur
					menuEdition[ALLER_ERREUR].setEnabled(false);
					for (int i=VOIR_ALGO;i<=VOIR_VAR;i++)
						menuAnalyse[i].setEnabled(false);
					menuEdition[ALLER_ERREUR].setEnabled(false);
					for (int i=ALLER_LIGNE;i<=INSERER_TANT_QUE;i++)
						menuEdition[i].setEnabled (true);
					menuTraduction[VOIR_TRADUCTION].setEnabled(false);
					fichierTraduit=null;
					// Autorise les modifications dans l'diteur
					editeur.setEditable(true);
				}
			}
		}
		else if (cmd.equals("enregistrer")) {
			enregistrer();
		}
		else if (cmd.equals("enregistrer_sous")) {
			enregistrer_sous ();
		}
		else if (cmd.equals("nouveau")) {
			if (modifie==true) {
				int choix = JOptionPane.showConfirmDialog(this, "L'algorithme a t modifi.\nVoulez-vous l'enregistrer ?","Voulez-vous sauvegarder ?",JOptionPane.YES_NO_OPTION);
				if (choix==JOptionPane.YES_OPTION) {
					if (fichierAlg!=null)
			 			enregistrer_sous ();
					else
						enregistrer ();
				}
			}	
			fichierAlg=null;
			// Efface le texte contenu dans l'diteur
			editeur.setText ("");
			// Met  jour la barre d'tat
			barreDeStatut.setNomFichier("Nouvel algorithme");
			modifie=false;
		}
		else if (cmd.equals("inserer_affectation")) {
			Affectation affectation = new Affectation ();
			AffectationDlg dialogue = new AffectationDlg (this, "Insrer une affectation", affectation);
			if (affectation.isValide())
				editeur.insert ("affecter "+affectation.getPartieGauche()+" <-- "+affectation.getPartieDroite(),editeur.getCaretPosition());
		}
		else if (cmd.equals("inserer_commentaire")) {
			Commentaire commentaire = new Commentaire ();
			CommentaireDlg dialogue = new CommentaireDlg (this, "Insrer une affectation", commentaire);
			if (commentaire.isValide())
				editeur.insert ("# "+ commentaire.getCommentaire(),editeur.getCaretPosition());
		}
		else if (cmd.equals("inserer_boucle_pour")) {
			BouclePour bouclePour = new BouclePour ();
			BouclePourDlg dialogue = new BouclePourDlg (this, "Insrer une boucle pour", bouclePour);
			if (bouclePour.isValide()) {
				String var=bouclePour.getVariable();
				String valInit=bouclePour.getValeurInitiale();
				String valEnd=bouclePour.getValeurFinale();
				editeur.insert ("pour "+var+" de "+valInit+" a "+valEnd+"\n\nfin_pour  #  "+var+ " de "+valInit+" a "+valEnd ,editeur.getCaretPosition());
			}
		}
		else if (cmd.equals("inserer_test_si")) {
			TestSi testSi = new TestSi ();
			TestSiDlg dialogue = new TestSiDlg (this, "Insrer un test si...alors...sinon", testSi);
			if (testSi.isValide()) {
				String buffer = new String ("si "+testSi.getCondition()+"\nalors\n\n");
				if (testSi.getSinon())
					buffer+="sinon\n\nfin_si  # "+testSi.getCondition();
				else
					buffer+="fin_si  # "+testSi.getCondition();
				editeur.insert (buffer,editeur.getCaretPosition());
			}
		}
		else if (cmd.equals("inserer_boucle_tant_que")) {
			BoucleTantQue boucleTantQue = new BoucleTantQue ();
			BoucleTantQueDlg dialogue = new BoucleTantQueDlg (this,"Insrer une boucle tant que",boucleTantQue);
			if (boucleTantQue.isValide())
				editeur.insert ("tant_que "+boucleTantQue.getCondition()+"\n\nfin_tant_que  # "+boucleTantQue.getCondition(),editeur.getCaretPosition());
		}
		else if (cmd.equals("aller_ligne")) {
			Ligne ligne = new Ligne();
			LigneDlg dialogue = new LigneDlg (this,"Aller  la ligne n",ligne,editeur.getLineCount());
			if (ligne.isValide()) {
				try {
					int debut=editeur.getLineStartOffset (ligne.getNumero()-1);
					int fin=editeur.getLineEndOffset (ligne.getNumero()-1);
					editeur.setCaretPosition(fin);
					editeur.moveCaretPosition (debut);
				}
				catch (BadLocationException except) {
					JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant le dplacement de l'diteur sur la ligne demande","Erreur",JOptionPane.ERROR_MESSAGE);
				}
				catch (IllegalArgumentException except) {
					JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant la mise en vidence de la ligne demande","Erreur",JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		else if (cmd.equals("lancer_analyse")) {
			if (modifie) {
				if (fichierAlg==null) {
					enregistrer_sous ();
					if (fichierAlg==null)
						// Algorithme non sauv, on ne peut l'analyser
						return;
				}
				else 
					enregistrer();
			} else if (fichierAlg==null) {
				JOptionPane.showMessageDialog(this,"Impossible d'analyser un algorithme vide","Erreur",JOptionPane.WARNING_MESSAGE);
				return;
			}
			try{
				Process p = Runtime.getRuntime().exec("galg -a "+fichierAlg);
				int res=p.waitFor();           // Attend la fin de la traduction
				if (res!=0) {
					JOptionPane.showMessageDialog(this,"GALG a dtect des erreurs dans votre algorithme","Rapport d'analyse",JOptionPane.INFORMATION_MESSAGE);
					// On rend accessible le saut  la premire erreur
					menuEdition[ALLER_ERREUR].setEnabled(true);
				}
				else 
					JOptionPane.showMessageDialog(this,"GALG n'a dtect aucune erreur dans votre algorithme","Rapport d'analyse",JOptionPane.INFORMATION_MESSAGE);
				voirRes ();
				menuTraduction[VOIR_TRADUCTION].setEnabled(false);
				fichierTraduit=null;
			}
			catch (IOException except) {
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'excution de GALG","Erreur",JOptionPane.ERROR_MESSAGE);
			}
			catch (InterruptedException except) {
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'excution de GALG","Erreur",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (cmd.equals("voir_algo")) {
			voirAlgo ();
		}
		else if (cmd.equals("voir_res")) {
			sauvegarde();
			voirRes ();		
		}
		else if (cmd.equals("voir_var")) {
			sauvegarde ();
			String Lvm = new String (fichierAlg.toString());
			Lvm=Lvm.replaceFirst("[.]alg",".lvm");
			// On affiche le fichier lvm
			File lvm=new File (Lvm);
			if (ouvrir (lvm)) {
				// Editeur en lecture seule
				editeur.setEditable(false);
				// On rend accessible les fonctions ncessaires du menu
				menuFichier[ENREGISTRER].setEnabled(false);
				menuFichier[ENREGISTRER_SOUS].setEnabled(false);
				menuAnalyse[VOIR_ALGO].setEnabled (true);
				menuAnalyse[VOIR_RES].setEnabled(true);
				menuAnalyse[VOIR_VAR].setEnabled(false);
				for (int i=ALLER_LIGNE;i<=INSERER_TANT_QUE;i++)
					menuEdition[i].setEnabled(false);
				if (fichierTraduit==null)
					menuTraduction[VOIR_TRADUCTION].setEnabled(false);
				else
					menuTraduction[VOIR_TRADUCTION].setEnabled(true);
				for (int i=ALLER_ERREUR;i<=INSERER_TANT_QUE;i++)
								menuEdition[i].setEnabled(false);
			}
			modifie=false;
		}
		else if (cmd.equals("aller_premiere_erreur")) {
			String Lst = new String (fichierAlg.toString());
			Lst=Lst.replaceFirst("[.]alg",".lst");
			// On ouvre le fichier lst
			File lst=new File (Lst);
			try {
				BufferedReader inputStream = new BufferedReader (new FileReader (lst));
				String buffer = inputStream.readLine();
				int index=0;
				boolean trouve=false;
				while ((buffer!=null) && (!trouve)) {
					if ((index=buffer.indexOf("***"))!=-1) {
						// La sous chane a t trouve
						trouve=true;
					}
					else 
						buffer = inputStream.readLine();
				}
				inputStream.close();
				if (trouve) {
					String ligneStr = buffer.substring (index+5,index+8);
					int nbligne = Integer.parseInt (ligneStr);
					// On charge l'algorithme si besoin est
					if (menuAnalyse[VOIR_ALGO].isEnabled()) {
						voirAlgo ();
					}
					int debut=editeur.getLineStartOffset (nbligne-1);
					int fin=editeur.getLineEndOffset (nbligne-1);
					editeur.setCaretPosition(fin);
					editeur.moveCaretPosition (debut);					
				}
				else
					JOptionPane.showMessageDialog(this,"Erreur non trouve","Erreur",JOptionPane.ERROR_MESSAGE);
			}
			catch (FileNotFoundException except) {
				// Impossible d'ouvrir le fichier .lst
				JOptionPane.showMessageDialog(this,"Impossible d'ouvrir "+lst.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
			catch (IOException except) {
				// Erreur pendant la lecture
				JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant la lecture du fichier"+lst.getName(),"Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
			catch (BadLocationException except) {
				// Erreur de placement du symbole d'insertion
				JOptionPane.showMessageDialog(this,"Erreur de localisation de la premire erreur dans l'algorithme","Erreur",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (cmd.startsWith ("traduire")) {
			int index = cmd.indexOf("_");
			// On rcupre le langage choisi
			String langage = cmd.substring (++index);
			try {
				sauvegarde ();
				if (fichierAlg==null) {   // Algorithme non sauvegard
					JOptionPane.showMessageDialog(this,"Impossible de traduire un algorithme vide","Erreur",JOptionPane.WARNING_MESSAGE);
					return;
				}
				Process p=Runtime.getRuntime().exec("galg -a "+fichierAlg+" -o "+langage);
				int res=p.waitFor();
				if (res==0) {
					String extension;
					String traduit = new String (fichierAlg.toString());
					if (langage.equals("dbase"))
						extension=new String ("prg");
					else if (langage.equals("pascal"))
						extension=new String ("pas");
					else if (langage.equals("rexx"))
						extension=new String ("rex");
					else if (langage.equals("perl"))
						extension=new String ("pl");
					else 
						extension=new String (langage);	
					traduit=traduit.replaceFirst("[.]alg","."+extension);
					fichierTraduit=new File (traduit);
					ouvrir (fichierTraduit);
					modifie=false;
					editeur.setEditable (false);
					JOptionPane.showMessageDialog(this,"GALG n'a dtect aucune erreur dans votre algorithme","Rapport d'analyse",JOptionPane.INFORMATION_MESSAGE);
					menuTraduction[VOIR_TRADUCTION].setEnabled(false);
					for (int i=VOIR_ALGO;i<=VOIR_VAR;i++)
						menuAnalyse[i].setEnabled(true);
					for (int i=ALLER_ERREUR;i<=INSERER_TANT_QUE;i++)
						menuEdition[i].setEnabled(false);
				}
				else {
					voirRes();					
					JOptionPane.showMessageDialog(this,"GALG a dtect des erreurs dans votre algorithme","Rapport d'analyse",JOptionPane.INFORMATION_MESSAGE);
				}
			}
			catch (IOException except) {
				JOptionPane.showMessageDialog(this,"Impossible d'xcuter GALG","Erreur d'E/S",JOptionPane.ERROR_MESSAGE);
			}
			catch (InterruptedException except) {
				// Echec de waitFor()
				JOptionPane.showMessageDialog(this,"Erreur lors de l'xcution de GALG","Erreur",JOptionPane.ERROR_MESSAGE);
			}
		}
		else if (cmd.equals("voir_traduction")) {
			sauvegarde();
			ouvrir (fichierTraduit);
			modifie=false;
			for (int i=ALLER_ERREUR;i<=INSERER_TANT_QUE;i++)
				menuEdition[i].setEnabled(false);
			menuTraduction[VOIR_TRADUCTION].setEnabled(false);
		}
		else if (cmd.startsWith("executer")) {
			sauvegarde();
			if (fichierAlg==null)  {   // Algorithme non sauvegard
				JOptionPane.showMessageDialog(this,"Impossible d'excuter un algorithme vide","Erreur",JOptionPane.WARNING_MESSAGE);	
				return;
			}
			Parametre param = new Parametre();
			ParametreDlg dialogue = new ParametreDlg (this, "Saisie des paramtres", param);
			if (!param.getValide()) {
				return;
			}
			int index = cmd.indexOf("_");
			// On rcupre le langage choisi
			String langage = cmd.substring (++index);
			Properties prop = System.getProperties();
			if ((prop.getProperty("os.name")).equals("Linux")) {
				String command = new String ("galg -a "+fichierAlg+" -o "+langage+" -x "+param.getParams()+"\n");
				String script = new String ("#!/bin/sh\n"+command+"echo \"Appuyez sur la touche Entre\"\nread\n");
				File scriptFile = new File ("galgExec.sh");
				try {
					BufferedWriter stream = new BufferedWriter (new FileWriter (scriptFile));
					stream.write (script);
					stream.flush();
					stream.close();
					Process p=Runtime.getRuntime().exec("xterm -e /bin/sh galgExec.sh");
					p.waitFor();
				}
				catch (IOException except) {
					JOptionPane.showMessageDialog(this,"Impossible d'ouvrir un terminal pour l'excution","Erreur",JOptionPane.ERROR_MESSAGE);
				}
				catch (InterruptedException except) {
					JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'excution","Erreur",JOptionPane.ERROR_MESSAGE);
				}
			}
			else {
				// Windows
				String command = new String ("call galg -a "+fichierAlg+" -o "+langage+" -x\r\n");
				String script = new String (command+"echo \"Appuyez sur une touche ...\"\npause\r\n");
				File scriptFile = new File ("galgExec.bat");
				try {
					BufferedWriter stream = new BufferedWriter (new FileWriter (scriptFile));
					stream.write (script);
					stream.flush();
					stream.close();
					Process p=Runtime.getRuntime().exec("COMMAND.COM /C galgExec.bat");
					p.waitFor();
				}
				catch (IOException except) {
					JOptionPane.showMessageDialog(this,"Impossible d'ouvrir un terminal pour l'excution","Erreur",JOptionPane.ERROR_MESSAGE);
				}
				catch (InterruptedException except) {
					JOptionPane.showMessageDialog(this,"Une erreur est survenue pendant l'excution","Erreur",JOptionPane.ERROR_MESSAGE);
				}
			}
		}
	}
}
